;**************************************************************************
;******** TTTTTT EEEEEE PPPPP  LL      OOOO  MM   MM EEEEEE RRRRR  ********
;********   TT   EE     PP  PP LL     OO  OO MMM MMM EE     RR  RR ********
;********   TT   EEEE   PPPPP  LL     OO  OO MM M MM EEEE   RRRRR  ********
;********   TT   EE     PP     LL     OO  OO MM   MM EE     RR RR  ********
;********   TT   EEEEEE PP     LLLLLL  OOOO  MM   MM EEEEEE RR  RR ********
;**************************************************************************
;********  Digitalni teplomer s cidlem DALLAS - DS1721S (12-bits)  ********
;**************************************************************************
;********  Vypracoval: Radek Zeman                                 ********
;********  Pouzite obvody: AT89C2051, DS1721S, M5451, TL7705A      ********
;**************************************************************************

;Ucc
;Gnd
;Xtal1
;Xtal2
;RESET
;IE0	(0003h) => Reset Maximalni/Minimalni teploty = Spinac
;P1.7	Displey => Seriova Data
;P1.6	Displey => Clock
;P1.5	Teplomer => Seriova Data
;P1.4	Teplomer => Clock
;P3.0	Dioda, zda je nyni videt Aktualni teplota nebo Maximum
;P1.3	Zobraz Maximalni teplotu = Spinac
;P3.1	Dioda, zda je nyni videt Aktualni teplota nebo Minimum
;P1.2	Zobraz Minimalni teplotu = Spinac
;---------------------------------------------------------------

DIS_DAT		BIT	P1.7
DIS_CLK		BIT	P1.6
TEP_DAT		BIT	P1.5
TEP_CLK		BIT	P1.4
DIODA_X		BIT	P3.0
ZOB_X		BIT	P1.3
DIODA_M		BIT	P3.1
ZOB_M		BIT	P1.2
A_7		BIT	0E7h
ZNAMENKO	BIT	020h	;Znamenko teploty
SMAZ		BIT	021h	;Zda se ma smazat ci ne

org 30h

DISP0:		DS	1h	; +-   , Staci jen poslat na
DISP1: 		DS	1h	; 1 rad, sbernici
DISP2: 		DS 	1h	; 2 rad, s des. teckou
DISP3: 		DS 	1h	; 1 rad, za des. teckou
HODDISP0:	DS	1h	;0-1 (8) Musi se dekodovat na
HODDISP1:	DS	1h	;0-9 (8) znaky, ktere se upravi
HODDISP2:	DS	1h	;0-9 (8) a presunou do DISPX
DESETINY:	DS	1h	;0-9 (8)
MAXMSB:		DS	1h	;Maximalni teplota MSB
MAXLSB:		DS	1h	;Maximalni teplota LSB
MINMSB:		DS	1h	;Minimalni teplota MSB
MINLSB:		DS	1h	;Minimalni teplota LSB
R1_ZALOHA:	DS	1h	;Zaloha teploty MSB
R2_ZALOHA:	DS	1h	;Zaloha teploty LSB
;----------------------------------------------------------------
	org	0000h			;Hned po resetu
	ljmp	MAIN			;Skoc na MAIN

	org	0003h			;Reset teploty
	setb	SMAZ			;Nastaveni, aby smazal	
	RETI				;Navrat z preruseni

	org	000Bh			;Citac 0	
	setb	C			;Nastav Carry
	RETI				;Navrat z preruseni

;--------------------------------
	org	000Dh
MAIN:	mov	IE,#10000001b		;Pouze externi preruseni
	mov	TCON,#00000101b		;Sestupna hrana
	setb	DIODA_X			;Aby nesvitila dioda MAX
	setb	DIODA_M			;Aby nesvitila dioda MIN
	clr	SMAZ			;Aby se nemazala teplota
	mov	SP,#08h			;Nastaveni zasobniku R0-2
	mov	MAXMSB,#0C9h		;Nastaveni MSB = -55
	mov	MINMSB,#07Dh		;Nastaveni MSB = +125
	acall	SMAZDIS			;Smaze display
		
	acall	START			;START
	mov	A,#10010000b		;Adresa teplomeru + zapis
	acall	ADR_COM			;ADRESA-KOMUNIKACE
	acall	ACK			;ACK
	mov	A,#51h			;A=inicializace prevodu		
	acall	ADR_COM			;Zacne prevadet => 1.2s
	acall	ACK			;ACK
	acall	STOP			;STOP
					;Inicializace teplomeru
	mov	DISP0,#00001100b	;I
	mov	DISP1,#00101010b	;n
	mov	DISP2,#00001000b	;i
	mov	DISP3,#00011010b	;c
	acall	POSLIDIS		;SEGMENTY=>M5451

;--------------------------------
OPAKUJ:	
	mov	R4,#0h			;Vynuluj R4
	mov	R5,#16d			;Kolikrat pricitat

OPP:	mov	R6,#12d			;Kolik hodnot prumeruji
OP:	mov	R3,#1d			;1x 0.1s
	acall	PAUZA			;=0.1s

;================================
	jnb	ZOB_X,Z_MAX		;ZOB_X=0 => Z_MAX
	ajmp	NE_ZOBX			;Nezmacknuto => NE_ZOBX
Z_MAX:		
	clr	SMAZ			;Nuluj SMAZ
	setb	DIODA_M			;Napeti, aby nesvitila	
	mov	R1,MAXMSB		;Presun MAXMSB=>R1
	mov	R2,MAXLSB		;Presun MAXLSB=>R2
	acall	ZLATA_3			;DEKODER,PREVOD,POSLIDIS

	clr	DIODA_X			;Uzemni, aby svitila MAX
	mov	R0,#30d			;Doba sviceni Diody
SVIT:					;Nech to chvili na displ.
	mov	R3,#1d			;1*0.1=0.1s
	acall	PAUZA			;Cekani urcitou dobu
	jbc	SMAZ,BLIK		;SMAZ=1 => BLIK a SMAZ=0
	jnb	ZOB_M,Z_MIN		;ZOB_M=0 => Z_MIN
	djnz	R0,SVIT			;R0 pocet opakovani
	setb	DIODA_X			;Napeti, aby nesvitila	
	ajmp	NORMA			;Preskoc blikani

	mov	R0,20d			;Pocet blikani diody
BLIK:	cpl	DIODA_X			;Neguj stav diody
	mov	R3,#2d			;0.1*2=0.2s
	acall	PAUZA			;Cekani urcitou dobu
	djnz	R0,BLIK			;R0=R0-1, R0==0  => BLIK

	acall	OB_MAX			;Obnov maximalni hodnotu
	setb	DIODA_X			;Napeti, aby nesvitila	
	clr	SMAZ			;Nuluj SMAZ

NORMA:	acall	OBNOV_RR		;Obnov R1, R2
	acall	ZLATA_3			;DEK-PRE-POSDIS
;-------------------------------
NE_ZOBX:
	jnb	ZOB_M,Z_MIN		;ZOB_M=0 => Z_MIN
	ajmp	NE_ZOBM			;Nezmacknuto => NE_ZOBM
Z_MIN:		
	clr	SMAZ			;Nuluj SMAZ
	setb	DIODA_X			;Napeti, aby nesvitila	
	mov	R1,MINMSB		;Presun MINMSB=>R1
	mov	R2,MINLSB		;Presun MINLSB=>R2
	acall	ZLATA_3			;DEKODER,PREVOD,PODLIDIS

	clr	DIODA_M			;Uzemni, aby svitila MAX
	mov	R0,#30d			;Doba sviceni Diody
SVIT2:					;Nech to chvili na displ.
	mov	R3,#1d			;1*0.1=0.1s
	acall	PAUZA			;Cekani urcitou dobu
	jbc	SMAZ,BLIK2		;SMAZ=1 => BLIK2 a SMAZ=0
	jnb	ZOB_X,Z_MAX		;ZOB_X=0 => Z_MAX
	djnz	R0,SVIT2		;R0 pocet opakovani
	setb	DIODA_M			;Napeti, aby nesvitila	
	ajmp	NORMA2			;Skoc na OPAKUJ

	mov	R0,20d			;Pocet blikani diody
BLIK2:	cpl	DIODA_M			;Neguj stav diody
	mov	R3,#2d			;0.1*2=0.2s
	acall	PAUZA			;Cekani urcitou dobu
	djnz	R0,BLIK2		;R0=R0-1, R0==0  => BLIK2

	acall	OB_MIN			;Obnov minimalni hodnotu
	setb	DIODA_M			;Napeti, aby nesvitila	
	clr	SMAZ			;Nuluj SMAZ

NORMA2:	acall	OBNOV_RR		;Obnov R1, R2
	acall	ZLATA_3			;DEK-PRE-POSDIS
;================================
NE_ZOBM:

	djnz	R6,OP			;Dokud neni 12x hodnota
	
	acall	TEPLOTA			;Cteni teploty
	mov	A,R2			;A=R2=LSB
	swap	A			;XXXX0000 => 0000XXXX
	add	A,R4			;A=A+R4
	mov	R4,A			;Zaloha vysledku A
	
	djnz	R5,OPP			;Dokud nebude 16x teplota

	anl	A,#11110000b		;Vydeleni 16 a vymaskovani
	mov	R2,A

	acall	MAXIMUM			;Nastaveni maxima
	acall	MINIMUM			;Nastaveni minima

	mov	R1_ZALOHA,R1		;Zaloha R1
	mov	R2_ZALOHA,R2		;Zaloha R2

	acall	ZLATA_3			;DEKODER,PREVOD,POSLIDIS
	ajmp	OPAKUJ			;Skoc na OPAKUJ

;-----------------------------------------------
;----------------------------------------------------------------
TEPLOTA:
	acall	START			;START
	mov	A,#10010000b		;Adresa teplomeru + zapis
	acall	ADR_COM			;ADRESA-KOMUNIKACE
	acall	ACK			;ACK
	mov	A,#0AAh			;Cteni teploty
	acall	ADR_COM			;ADRESA-KOMUNIKACE
	acall	ACK			;ACK

	setb	TEP_DAT			;R_START
	setb	TEP_CLK			;Restart komunikace
	clr	TEP_DAT	
	clr	TEP_CLK

	mov	A,#10010001b		;Adresa teplomeru + cteni
	acall	ADR_COM			;ADRESA-KOMUNIKACE
	acall	ACK			;ACK
	acall	CTI			;CTI hodnotu
	mov	R1,A			;MSB=A=R1
	
	clr	TEP_DAT			;M_ACK
	setb	TEP_CLK			;Potvrzeni prijmu 8051
	clr	TEP_CLK
	setb	TEP_DAT			;Aby 8051 cetla

	acall	CTI			;CTI hodnotu
	mov	R2,A			;LSB=A=R2

	setb	TEP_DAT			;Potvrzeni prijmu 8051
	setb	TEP_CLK			;Konec komunikace
	clr	TEP_CLK	

	acall	STOP			;STOP komunikace
	RET
;----------------------------------------------------------------
	;Prevod MSB a LSB na CISLA
	;Naplni ZNAMENKO,HODXX,DESETINY vstup R1(MSB),R2(LSB)
DEKODER:
	mov	A,R1			;Presun MSB do A
	setb	ZNAMENKO		;Predpoklad kladna teplo.	
	jnb	A_7,KLADNE		;Zjisteni zda je zaporne
	
	clr	ZNAMENKO		;Zaporna teplota,0
	cpl	A			;Negace MSB
	mov	R1,A			;Presun uprav. MSB nazpet
	mov	A,R2			;Presun LSB do A
	cpl	A			;Negace LSB
	anl	A,#11110000b		;Vymaskuj pos. 4 bity AND
	add	A,#00010000b		;Pricteni jednicky=dopln.
	mov	R2,A			;Vysledna teplota LSB
	mov	A,R1			;Abych mohl pricist C
	addc	A,#00000000b		;Prictu pouze C
	mov	R1,A			;Vysledna teplota MSB
KLADNE:					;HODDISP0 uz naplnen
	mov	A,R1			;Delim MSB
	mov	B,#100d			;B=100
	div	AB			;A/B
	mov	HODDISP0,A		;Stovky
	mov	A,B			;Delim zbytek po 100
	mov	B,#10d			;A=10
	div	AB			;A/B
	mov	HODDISP1,A		;Desitky	
	mov	HODDISP2,B		;Zbytek (jednotky)

	mov	A,R2			;Presun LSB do A
	swap	A			;=rrrrrrrr A
	mov	DESETINY,A		;Presun A do DESETINY
	RET

;----------------------------------------------------------------
	;Prevod CISLA na Displey nozicky (naplneni DISPx)
PREVOD:
	jnb	ZNAMENKO,ZAPTEP		;Zaporna teplota => ZAPTEP
	mov	A,HODDISP0		;Zde kladne, kvuli skoku
	mov	DISP0,#00000000b	;Predpoklad NIC
	cjne	A,#1h,B_NIC		;A nerovna 1 => B_NIC
	mov	DISP0,#01100000b	;A=1
	ajmp	B_NIC			;Preskoc zapornou teplotu
ZAPTEP: mov	DISP0,#00000010b	;Minus
B_NIC:
	mov	A,HODDISP1		;Display 1 s redukci 0
	cjne	A,#0h,N_NULA		;Neni 0 skoc na N_NULA
	mov	DISP1,#00000000b	;Presun do DISP1
	ajmp	PRESCI			;Skoc na PRESCI
N_NULA: mov	DPTR,#ZNAK		;Dekoder ZNAKU
	movc	A,@A+DPTR
	mov	DISP1,A
PRESCI:
	mov	A,HODDISP2		;Display 2 s Teckou
	mov	DPTR,#ZNAK		;Dekoder ZNAKU
	movc	A,@A+DPTR
	orl	A,#00000001b		;Tecka
	mov	DISP2,A

	mov	A,DESETINY		;Display 3, desetina cast
	mov	DPTR,#DZNAK		;Dekoder ZNAKU
	movc	A,@A+DPTR
	mov	DISP3,A
	RET

;----------------------------------------------------------------
	;Smaze Displey
SMAZDIS:	
	setb	DIS_DAT			;Nastaveni prenosu,
	setb	DIS_CLK			;START bit
	clr	DIS_CLK

	clr	DIS_DAT
	mov	R0,#35d			;Odesli 37 prazdnych bitu
MAZ:	setb	DIS_CLK
	clr	DIS_CLK
	djnz	R0,MAZ			;R0=R0-1, R0==0 skoc MAZ
	RET

;----------------------------------------------------------------
	;Posli Displey na Sbernici, NICI DISPx
POSLIDIS:
	setb	DIS_DAT			;Nastaveni prenosu
	setb	DIS_CLK			;START bit
	clr	DIS_CLK

	mov	A,DISP0			;Poslani 0 displeje,
	mov	R0,#07h			;Znamenko
D0:	rlc	A			;(7 bitu)
	mov	DIS_DAT,C
	setb	DIS_CLK
	clr	DIS_CLK
	djnz	R0,D0			;R0=R0-1, R0==0 skoc D0

	mov	A,DISP1			;Poslani 1 displeje,
	mov	R0,#07h			;(7 bitu)
D1:	rlc	A
	mov	DIS_DAT,C
	setb	DIS_CLK
	clr	DIS_CLK
	djnz	R0,D1			;R0=R0-1, R0==0 skoc D1

	mov	A,DISP2			;Poslani 2 displeje,
	mov	R0,#08h			;(8 bitu)
D2:	rlc	A
	mov	DIS_DAT,C
	setb	DIS_CLK
	clr	DIS_CLK
	djnz	R0,D2			;R0=R0-1, R0==0 skoc D2

	mov	A,DISP3			;Poslani 3 displeje,
	mov	R0,#07h			;(7 bitu)
D3:	rlc	A
	mov	DIS_DAT,C
	setb	DIS_CLK
	clr	DIS_CLK
	djnz	R0,D3			;R0=R0-1, R0==0 skoc D3

	setb	DIS_DAT			;Aby DISP=0, DATA=1
	mov	R0,#12d			;vyvodu (6 bitu)
VOLNO:	cpl	DIS_CLK
	djnz	R0,VOLNO		;R0=R0-1, R0==0 => VOLNO
	RET

;----------------------------------------------------------------
	;Start prenosu
START:	
	setb	TEP_CLK			;Sbernice volna
	setb	TEP_DAT
	clr	TEP_DAT			;Start bit	
	clr	TEP_CLK
	RET	

;----------------------------------------------------------------
	;Adresa teplomeru a command byte
ADR_COM:				;A=Adresa teplo. + zapis
	mov 	R0,#8d			;R0=8
ADC:	rlc	A			;Vyslani Adresy
	mov	TEP_DAT,C
	setb	TEP_CLK
	clr	TEP_CLK
	djnz	R0,ADC			;R0=R0-1, R0==0 skoc ADC
	RET

;----------------------------------------------------------------
	;Potvrzeni prijmu teplomeru
ACK:	
	setb	TEP_DAT			;Aby 8051 cetla
	setb	TEP_CLK			;Potvrzeni prijmu v C
	mov	C,TEP_DAT
	clr	TEP_CLK
       	jnc	NECHYBA
					;Chyba prijmu u teplomeru
	mov	DISP0,#10011110b	;E
	mov	DISP1,#00001010b	;r
	mov	DISP2,#00001010b	;r
	mov	DISP3,#00111010b	;o
	acall	POSLIDIS		;SEGMENTY=>M5451

	mov	R3,#30d			;0.1*30=3s
	acall	PAUZA			;Cekani urcitou dobu
	ajmp	MAIN			;Skoc na MAIN
NECHYBA:
	RET	

;----------------------------------------------------------------
	;Stop komunikace
STOP:	
	setb	TEP_CLK
	setb	TEP_CLK
	setb	TEP_DAT
	RET

;----------------------------------------------------------------
	;Cteni dat z teplomeru do 8051
CTI:	
	mov	R0,#8h			;V A bude 8 bitu po cteni
	clr	A			;Vynuluj A
C1:	setb	TEP_CLK
	mov	C,TEP_DAT
	rlc	A			;Leva rotace pres Carry
	clr	TEP_CLK
	djnz	R0,C1			;R0=R0-1, R0==0 skoc C1
	RET

;----------------------------------------------------------------
	;Naplneni maximalni a minimalni teploty
	;Vstup  = R1(MSB),R2(LSB),MAXMSB,MAXLSB
	;Vystup = MAXMSB,MAXLSB (MAXIMUM)
MAXIMUM:			
	;Rozvetveni
	mov	A,MAXMSB
	jnb	A_7,A_KLADNE
	mov	A,R1			
	jbc	A_7,AB_KLADNE
	mov	MAXMSB,R1		;Zde je MAXMSB-, R1+
	mov	MAXLSB,R2
	RET
A_KLADNE:
	mov	A,R1
	jbc	A_7,RET_M		;Zde je MAXMSB+, R1-
AB_KLADNE:				;Zde je MAXMSB+, R1+
	acall	VETSI			;Zde je MAXMSB-, R1-
RET_M:	RET

;----------------------------------------------------------------	
	;Zjisteni co je vetsi a co je mensi
VETSI:	
	clr	C
	mov	A,MAXLSB
	subb	A,R2
	mov	A,MAXMSB
	subb	A,R1
	jnc	K_RET			;MAXMSB,MAXLSB vetsi
	mov	MAXMSB,R1		;Namerena vetsi
	mov	MAXLSB,R2
K_RET:	RET

;----------------------------------------------------------------
	;Naplneni minimalni teploty
	;Vstup	= R1(MSB), R2(LSB), MINMSB, MINLSB
	;Vystup = MINMSB, MINLSB (MINIMUM)
MINIMUM:
	;Rozvetveni
	mov	A,MINMSB
	jnb	A_7,A_KLA
	mov	A,R1
	jbc	A_7,AB_KLA
	RET				;Zde je MINMSB-, R1-
A_KLA:
	mov	A,R1
	jnb	A_7,AB_KLA		;Zde je MINMSB+, R1-
	mov	MINMSB,R1
	mov	MINLSB,R2
	RET
AB_KLA:					;Zde je MINMSB+,R1+
	acall	MENSI
	RET				;Zde je MINMSB-, R1+

;----------------------------------------------------------------
	;Zjisteni co je mensi
MENSI:
	clr	C
	mov	A,MINLSB
	subb	A,R2
	mov	A,MINMSB
	subb	A,R1
	jc	NAM_VM			;MINMSB, MINLSB vetsi
	mov	MINMSB,R1
	mov	MINLSB,R2
NAM_VM:	RET				;Namerena vetsi

;----------------------------------------------------------------
	;Cekani urcitou dobu (min. 0.1s)
	;Musi predchazet naplneni R3=pocet 0.1s	
PAUZA:
	mov	IE,#10000011b		;Povoleni citace 0 a IE0
	mov	TMOD,#00000001b		;16 bit citac 0

NECASX:	mov	TH0,#7Dh		;32202=>33333*3us
	mov	TL0,#0CAh		;=0.1s
	setb	TR0			;Spusti citac 0

	clr	C			;Nuluj C
CASX:	jnc	CASX			;Dokud neni INT => CAS
	djnz	R3,NECASX		;R3 = pocet opakovani
	clr	TR0			;Zastavi citac 1

	mov	IE,#10000001b		;Pouze Externi preruseni
	RET

;----------------------------------------------------------------
ZLATA_3:
	acall	DEKODER			;MSB,LSB=>CISLA
	acall	PREVOD			;CISLA=>SEGMENTY
	acall	POSLIDIS		;SEGMENTY=>M5451
	RET

;----------------------------------------------------------------
OBNOV_RR:
	mov	R1,R1_ZALOHA
	mov	R2,R2_ZALOHA
	RET

;----------------------------------------------------------------
OB_MAX:
	mov	MAXMSB,R1_ZALOHA
	mov	MAXLSB,R2_ZALOHA
	RET

;----------------------------------------------------------------
OB_MIN:
	mov	MINMSB,R1_ZALOHA
	mov	MINLSB,R2_ZALOHA
	RET

;----------------------------------------------------------------
ZNAK:	DB	11111100b	;0 03	Tabulka cislic
	DB	01100000b	;1 9F
	DB	11011010b	;2 25
	DB	11110010b	;3 0D
	DB	01100110b	;4 99
	DB	10110110b	;5 49
	DB	10111110b	;6 41
	DB	11100000b	;7 1F
	DB	11111110b	;8 01
	DB	11110110b	;9 09

DZNAK:	DB	11111100b	;0 03	Tabulka cislic teplomeru
	DB	11111100b	;0 03
	DB	01100000b	;1 9F
	DB	11011010b	;2 25
	DB	11011010b	;2 25
	DB	11110010b	;3 0D
	DB	11110010b	;3 0D
	DB	01100110b	;4 99
	DB	10110110b	;5 49
	DB	10110110b	;5 49
	DB	10111110b	;6 41
	DB	11100000b	;7 1F
	DB	11100000b	;7 1F
	DB	11111110b	;8 01
	DB	11111110b	;8 01
	DB	11110110b	;9 09
	
END